local folder, core = ...
_G["_CLV"] = core

local curseVersion = GetAddOnMetadata(folder, "X-Curse-Packaged-Version")
local curseRevision = GetAddOnMetadata(folder, "X-Curse-Packaged-Revision") -- 11
if curseVersion and curseRevision and curseVersion ~= "r"..curseRevision then
	curseVersion = curseVersion.." (r"..curseRevision..")" --alpha versions will show "r99", tagged versions will show "1.2.3beta (r99)"
end

core.title		= GetAddOnMetadata(folder, "Title")
core.version	= curseVersion or "[dev]"
core.titleFull	= core.title.." "..(curseVersion and "v"..tostring(core.version) or tostring(core.version))
core.addonDir = "Interface\\AddOns\\"..folder.."\\"

LibStub("AceAddon-3.0"):NewAddon(core, folder, "AceConsole-3.0", "AceHook-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale(folder, true)

--~ local regEvents = {
--~ 	"ACTIONBAR_SLOT_CHANGED",
--~ 	"PLAYER_REGEN_DISABLED",
--~ 	"PLAYER_REGEN_ENABLED",
--~ }

core.defaultSettings = {
	profile = {},
	factionrealm = {}, --Used for cached prices.
}


local tostring = tostring
local chatFrame = _G["ChatFrame1"]

local DEBUG = false
--[===[@debug@
DEBUG = true
--@end-debug@]===]
local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.
local colouredName		= "|cff7f7f7f{|r|cffff0000CLV|r|cff7f7f7f}|r "
local function echo(...)
	local tbl  = {...}
	local msg = tostring(tbl[1])
	for i=2,#tbl do 
		msg = msg..strWhiteBar..tostring(tbl[i])
	end

	local cf = chatFrame
	if cf then
		cf:AddMessage(colouredName..msg,.7,.7,.7)
	end
end
core.echo = echo

local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
-----------------------------
local function Debug(from, ...)	--
-- simple print function.	--
------------------------------
--~ 	if DEBUG == false then
--~ 		return 
--~ 	end
	if core.db.profile.debugMessages == false then
		return
	end
	local tbl  = {...}
	local msg = tostring(tbl[1])
	for i=2,#tbl do 
		msg = msg..strWhiteBar..tostring(tbl[i])
	end
	echo(strDebugFrom:format(from).." "..msg)
end
core.Debug = Debug

do
	local LibStub = LibStub
	function core:OnInitialize()
		self.db = LibStub("AceDB-3.0"):New("CLV_DB", core.defaultSettings, true) --'Default'
	
		self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
		self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
		self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
		self.db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
	
	--~ 	if db.global.lastVersion ~= self.version then
	--~ 		db.global.lastVersion = self.version
	--~ 		StaticPopup_Show(folder.."VersionChange", self.version);
	--~ 	end
--~ 		self:Hook("SecureHandlerSetFrameRef", true, nil, true)
	end
end

----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
--~ 	-- Shut down anything left from previous settings
	self:Disable()
--~ 	-- Enable again with the new settings
	self:Enable()
end

do
	local pairs = pairs
	function core:OnEnable()
--~ 		for i, event in pairs(regEvents) do 
--~ 			self:RegisterEvent(event)
--~ 		end
		core:RawHook("ChatFrame_OnEvent", true)
	end
end

local LOOT_ITEM = string.gsub(LOOT_ITEM, "%%s", "(.+)")												--"%s receives loot: %s."
local LOOT_ITEM_CREATED_SELF = string.gsub(LOOT_ITEM_CREATED_SELF, "%%s", "(.+)") 					--"You create: %s."
local LOOT_ITEM_CREATED_SELF_MULTIPLE = string.gsub(LOOT_ITEM_CREATED_SELF_MULTIPLE, "%%s", "(.+)")
local LOOT_ITEM_CREATED_SELF_MULTIPLE = string.gsub(LOOT_ITEM_CREATED_SELF_MULTIPLE, "%%d", "(.+)")	--"You create: %sx%d."
local LOOT_ITEM_MULTIPLE = string.gsub(LOOT_ITEM_MULTIPLE, "%%s", "(.+)")
local LOOT_ITEM_MULTIPLE = string.gsub(LOOT_ITEM_MULTIPLE, "%%d", "(.+)")							--"%s receives loot: %sx%d."
local LOOT_ITEM_PUSHED_SELF = string.gsub(LOOT_ITEM_PUSHED_SELF, "%%s", "(.+)")						--"You receive item: %s."
local LOOT_ITEM_PUSHED_SELF_MULTIPLE = string.gsub(LOOT_ITEM_PUSHED_SELF_MULTIPLE, "%%s", "(.+)")
local LOOT_ITEM_PUSHED_SELF_MULTIPLE = string.gsub(LOOT_ITEM_PUSHED_SELF_MULTIPLE, "%%d", "(.+)")	--"You receive item: %sx%d."
local LOOT_ITEM_SELF = string.gsub(LOOT_ITEM_SELF, "%%s", "(.+)")									--"You receive loot: %s."
local LOOT_ITEM_SELF_MULTIPLE = string.gsub(LOOT_ITEM_SELF_MULTIPLE, "%%s", "(.+)")
local LOOT_ITEM_SELF_MULTIPLE = string.gsub(LOOT_ITEM_SELF_MULTIPLE, "%%d", "(.+)")					--"You receive loot: %sx%d."


do
	local strChatIcon	= "|T%s:0:0:0:0|t"; --sending icon to local chat, CANT be sent threw chat channels to others.
	function core:ChatIcon(texture)
		return strChatIcon:format(texture)
	end
end

do
	local strsplit = strsplit
	local tonumber = tonumber
	local function ItemLinkToId(itemLink) -- Itemlink, returns the itemID
		if itemLink then
			local _, _, itemString = itemLink:find("^|c%x+|H(.+)|h%[.+%]")
			if itemString then
				local _, itemId = strsplit(":", itemString)
				return tonumber(itemId)
			end
		end
		return 0
	end

	core.iID = setmetatable({}, {
		__index = function(t,i)
		-- Get the value and stash it for later
		local v = ItemLinkToId(i)
		t[i] = v
		return v
		end
	})
end



do
	local DEFAULT_CHAT_FRAME = DEFAULT_CHAT_FRAME
	local GetItemIcon = GetItemIcon
	local tonumber = tonumber
	
	local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12
	local chatFrame
	
	local link2
	function core:ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
		chatFrame = frame:GetName() or DEFAULT_CHAT_FRAME;
		
		if event == "CHAT_MSG_LOOT" then
			for player, link, stack in arg1:gmatch(LOOT_ITEM_MULTIPLE) do
				if self.db.profile.addItemIcon == true then
					arg1 = arg1:gsub(": ", ": "..self:ChatIcon( GetItemIcon(link) ))
				end
				arg1 = arg1.." "..self:GetValueString(link, stack);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
				return
			end
		
			for player, link in arg1:gmatch(LOOT_ITEM) do
				if self.db.profile.addItemIcon == true then
					arg1 = arg1:gsub(": ", ": "..self:ChatIcon( GetItemIcon(link) ))
				end
				arg1 = arg1.." "..self:GetValueString(link);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
				return
			end

			for link, stack in arg1:gmatch(LOOT_ITEM_CREATED_SELF_MULTIPLE) do
				if self.db.profile.addItemIcon == true then
					arg1 = arg1:gsub(": ", ": "..self:ChatIcon( GetItemIcon(link) ))
				end
				arg1 = arg1.." "..self:GetValueString(link, stack);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
				return
			end
			
			for link in arg1:gmatch(LOOT_ITEM_CREATED_SELF) do
				if self.db.profile.addItemIcon == true then
					arg1 = arg1:gsub(": ", ": "..self:ChatIcon( GetItemIcon(link) ))
				end
				arg1 = arg1.." "..self:GetValueString(link);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
				return
			end

			for link, stack in arg1:gmatch(LOOT_ITEM_PUSHED_SELF_MULTIPLE) do
				if self.db.profile.addItemIcon == true then
					arg1 = arg1:gsub(": ", ": "..self:ChatIcon( GetItemIcon(link) ))
				end
				arg1 = arg1.." "..self:GetValueString(link, stack);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
				return
			end
			
			for link in arg1:gmatch(LOOT_ITEM_PUSHED_SELF) do
				if self.db.profile.addItemIcon == true then
					arg1 = arg1:gsub(": ", ": "..self:ChatIcon( GetItemIcon(link) ))
				end
				arg1 = arg1.." "..self:GetValueString(link);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
				return
			end
			
			for link, stack in arg1:gmatch(LOOT_ITEM_SELF_MULTIPLE) do
				if self.db.profile.addItemIcon == true then
					arg1 = arg1:gsub(": ", ": "..self:ChatIcon( GetItemIcon(link) ))
				end
				arg1 = arg1.." "..self:GetValueString(link, stack);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
				return
			end
			
			for link in arg1:gmatch(LOOT_ITEM_SELF) do
				if self.db.profile.addItemIcon == true then
					arg1 = arg1:gsub(": ", ": "..self:ChatIcon( GetItemIcon(link) ))
				end
				arg1 = arg1.." "..self:GetValueString(link);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
				return
			end
			

		end
		
		
		self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
	end
end