local folder, core = ...

local LibStub = LibStub
local Debug = core.Debug
local echo = core.echo

local L = core.L or LibStub("AceLocale-3.0"):GetLocale(folder, true)

local defaultSettings = core.defaultSettings
defaultSettings.profile.debugMessages			= false
defaultSettings.profile.pricingAlgorithm			= ""
defaultSettings.profile.addItemIcon			= true
defaultSettings.profile.colourBlindMode			= false
defaultSettings.profile.moneyMessage			= "!ALGORITHM: !VALUE"

--~ input

local coreOpts
do 
	local OnInitialize = core.OnInitialize
	function core:OnInitialize(...)
		if OnInitialize then OnInitialize(self, ...) end

		self:AddBlizzardOptions()
		self:CreateOptionsDisplay()
		
		self:RegisterChatCommand("clv", "ChatCommand")
	end
end

----------------------------------------------
function core:ChatCommand(input)	--
----------------------------------------------
	core:OpenOptionsFrame()
end

local name = function(info) 
	local key = info[#info]
	return L[key.."Name"] and L[key.."Name"]:format(core.db.profile[key]) or key
end
local desc = function(info) 
	local key = info[#info]
	return L[key.."Desc"] and L[key.."Desc"]:format(core.db.profile[key]) or key
end

function core:OpenOptionsFrame()
	LibStub("AceConfigDialog-3.0"):Open(self.title)
end


do
	local GENERAL = GENERAL
	local ENABLE = ENABLE
	
	function core:CreateOptionsDisplay()
		local db = core.db
		local options = {
			type = "group",
			name = core.title,
	--~ 		get = function( k ) return db[k.arg] end,
	--~ 		set = function( k, v ) db[k.arg] = v; end,
			
			get = function(info)
				local key = info[#info]
				return db.profile[key]
			end,
			set = function(info, v)
				local key = info[#info] 
				db.profile[key] = v
				self:Disable()
				self:Enable()
			end,
			
			
			args = {},
			plugins = {},
		}
		
		options.plugins["profiles"] = { profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db) }
	
		options.args.general = {
			type = "group",
			name = GENERAL,
			order = 1,
			disabled = function(info) 
				if info.type ~= "group" then
					return not core:IsEnabled()
				end
				return false
			end,
			args = {
			
				enable = {
					type = "toggle",	order	= 1,
					name	= ENABLE,
					desc	= desc, --L.enableDesc,
					set = function(info,val) 
						if val == true then
							core:Enable()
						else
							core:Disable()
						end
					
					end,
					get = function(info) return core:IsEnabled() end,
					disabled = false,
				},
				
				addItemIcon = {
					type = "toggle", order	= 10,
					name = name, desc = desc,
				},
				colourBlindMode = {
					type = "toggle", order	= 11,
					name = name, desc = desc,
				},
				moneyMessage = {
					type = "input", order	= 12,
					name = name, desc = desc,
					
				},
				
				debugMessages = {
					type = "toggle", order	= 90,
					name = name, desc = desc,
				},
				
			}
		}

		
		
		local config = LibStub("AceConfig-3.0")
	--~ 	local dialog = LibStub("AceConfigDialog-3.0")
		config:RegisterOptionsTable(core.title, options ) --
	--~ 	coreOpts = dialog:AddToBlizOptions(core.title, core.titleFull)
		
		LibStub("AceConfigDialog-3.0"):SetDefaultSize(core.title, 600, 400) --680
		
		self.options = options
		
		self:BuildAboutMenu()
	end
end


do
	local UnitName = UnitName
	local HideUIPanel = HideUIPanel
	local InterfaceOptionsFrame = InterfaceOptionsFrame
	local GameMenuFrame = GameMenuFrame
	local CreateFrame = CreateFrame
	local table_wipe = table.wipe
	local coreOpts
	function core:AddBlizzardOptions()
	-- I'm going to use AceConfigDialog's window to show my options. 
	-- But I still want my addon listed in Blizzard's options frame. 
	-- The only thing shown in the Blizzard options frame will be a button 
	-- to open AceConfigDialog's window.
	
		local blizOptions = {
			name = core.name,
			type = "group",
			args = {
			
				Desc = {
					type = "description",
					name = L.blizOptionsName:format(UnitName("player")),
					order = 1,
				},
				openOptionsFrame = {
					type = "execute",	order	= 2,
					name	= name, --L.openOptionsFrameName,
					desc	= desc, --L.openOptionsFrameDesc,
					func = function(info, v)
						HideUIPanel(InterfaceOptionsFrame)
						HideUIPanel(GameMenuFrame)
						
						--If I open the AceConfigDialog window now it will close once HideUIPanel fires. 
						-- So I'm making a frame to open it on the next frame refresh.
						local f = CreateFrame("Frame")
						f:SetScript("OnUpdate", function(this, elapsed) 
							core:OpenOptionsFrame()
							this:Hide()
							table_wipe(this)
						end)
					end,
				},
			}
		}
	
		local config = LibStub("AceConfig-3.0")
		local dialog = LibStub("AceConfigDialog-3.0")
		local blizName = core.title.."bliz"
		
		config:RegisterOptionsTable(blizName, blizOptions )
		coreOpts = dialog:AddToBlizOptions(blizName, core.titleFull)
	end
end


do
	local tostring = tostring
	local GetAddOnMetadata = GetAddOnMetadata
	local pairs = pairs
	function core:BuildAboutMenu()
		local options = self.options
		
		options.args.about = {
			type = "group",
			name = L.about,
			order = 99,
			args = {
			}
		}
		
		local fields = {"Author", "X-Category", "X-License", "X-Email", "Email", "eMail", "X-Website", "X-Credits", "X-Localizations", "X-Donate"}
		local haseditbox = {["X-Website"] = true, ["X-Email"] = true, ["X-Donate"] = true, ["Email"] = true, ["eMail"] = true}
	
		local fNames = {
			["Author"] = L.author,
			["X-License"] = L.license,
			["X-Website"] = L.website,
			["X-Donate"] = L.donate,
			["X-Email"] = L.email,
		}
		local yellow = "|cffffd100%s|r"
		
		
		
		options.args.about.args.title = {
			type = "description",
			name = yellow:format(L.title..": ")..core.title,
			order = 1,
		}
		options.args.about.args.version = {
			type = "description",
			name = yellow:format(L.version..": ")..core.version,
			order = 2,
		}
		options.args.about.args.notes = {
			type = "description",
			name = yellow:format(L.notes..": ")..tostring(GetAddOnMetadata(folder, "Notes")),
			order = 3,
		}
	
		for i,field in pairs(fields) do
			local val = GetAddOnMetadata(folder, field)
			if val then
				
				if haseditbox[field] then
					options.args.about.args[field] = {
						type = "input",
						name = fNames[field] or field,
						order = i+10,
						desc = L.clickCopy,
						width = "full",
						get = function(info)
							local key = info[#info]
							return GetAddOnMetadata(folder, key)
						end,	
					}
				else
					options.args.about.args[field] = {
						type = "description",
						name = yellow:format((fNames[field] or field)..": ")..val,
						width = "full",
						order = i+10,
					}
				end
		
			end
		end
	
		LibStub("AceConfig-3.0"):RegisterOptionsTable(core.title, options ) --
	end
end

do
	local pairs = pairs
	local tostring = tostring
	
	function core:BuildPricingOptions()
		local options = self.options
		
		local algorithms, algorithmsIndex, defaultAlgorithmName = self:GetAlgorithms()
		
		local values = {}
		for name, index in pairs(algorithmsIndex) do 
			values[name] = name
		end
		
		if self.db.profile.pricingAlgorithm == "" then
			self.db.profile.pricingAlgorithm = defaultAlgorithmName
		end
	
		if not values[self.db.profile.pricingAlgorithm] then
			self.db.profile.prevAlgorithm = self.db.profile.pricingAlgorithm
			self.db.profile.pricingAlgorithm = defaultAlgorithmName
			Debug("BuildPricingOptions", tostring(self.db.profile.pricingAlgorithm).." not found. Changing to "..tostring(defaultAlgorithmName))
			
		elseif self.db.profile.prevAlgorithm and values[self.db.profile.prevAlgorithm] then
			self.db.profile.pricingAlgorithm = self.db.profile.prevAlgorithm
			Debug("BuildPricingOptions", "Price algorithm returning to "..tostring(self.db.profile.prevAlgorithm))
			self.db.profile.prevAlgorithm = nil
			
		elseif self.db.profile.pricingAlgorithm == "VendorValue" and defaultAlgorithmName ~= "VendorValue" then
	--~ 		if not self.db.profile.prevAlgorithm then
				echo(L.changingPriceAlgorithm:format("VendorValue", tostring(defaultAlgorithmName)))
				self.db.profile.pricingAlgorithm = defaultAlgorithmName
	--~ 		end
		end
		
		
		options.args.pricingGroup = {
			type = "group",
			name = name, desc = desc,
			order = 1,
			args = {
	--~ 			Desc = {
	--~ 				type = "description",
	--~ 				name = L.pricingDesc,
	--~ 				order = 10,
	--~ 			},
				
				pricingAlgorithm = {
					type = "select",	order	= 13,
					name	= name, --L.pricingAlgorithmName,
					desc	= desc, --L.pricingAlgorithmDesc,
					values = values,
					
					set = function(info, v)
						local key = info[#info] 
						self.db.profile[key] = v
						self:FlushPriceData()
					end,
					
				},

				--[[
				filterByProfit = {
					type = "toggle",	order	= 24,
					name	= name,
					desc	= desc,
	--~ 				disabled =  function() return self.db.profile.ignoreInventoryGems == 0 end,
				},
				
				minProfitGold = {
					type = "range",	order	= 25,
					name	= name, --L.minProfitGoldName,
					desc	= desc, --L.minProfitGoldDesc,
					min 	= 0,
					max 	= 100,
					step	= 5,
					disabled =  function() return not self.db.profile.filterByProfit end,
				},
				minProfitPercentage = {
					type = "range",	order	= 26,
					name	= name, --L.minProfitPercentageName,
					desc	= desc, --L.minProfitPercentageDesc,
					min 	= 100,
					max 	= 200,
					step	= 5,
					disabled =  function() return not self.db.profile.filterByProfit end,
				},
			]]
			
			},
		}
		
		LibStub("AceConfig-3.0"):RegisterOptionsTable(core.title, options ) --
	end
end