--[[
	ChatLootValue: A chat message loot value addon.
	Copyright (c) 2011 Cyprias <Cyprias@gmail.com>
	All Rights Reserved.
]]

--[[
	Pricing system stolen from LilSparky's Workshop.
	http://www.wowace.com/addons/lil-sparkys-workshop/
]]

local folder, core = ...

local L = core.L or LibStub("AceLocale-3.0"):GetLocale(folder, true)

local table_insert = table.insert
local pricingSupportList = {}
local algorithms = {}
local algorithmsIndex = {}
local defaultAlgorithm
local defaultAlgorithmName
local Debug = core.Debug

local prev_OnEnable = core.OnEnable
function core:OnEnable()
	if prev_OnEnable then
		prev_OnEnable(self)
	end

	self:FlushPriceData()
	self:BuildPricingOptions()
end

do
	local pairs = pairs
	
	local OnInitialize = core.OnInitialize
	function core:OnInitialize()
		if OnInitialize then OnInitialize(self) end

		for _, value in pairs(pricingSupportList) do 
			if value.Test() then
				value.Init()
			end
		end
	end
end


function core:GetAlgorithms()
	return algorithms, algorithmsIndex, defaultAlgorithmName
end

function core:RegisterAlgorithm(name, func)
--~ 	Debug("RegisterAlgorithm", name, func)
	
	
	if name and func then

		local a = {}
		a.name = name
		a.func = func

		local i = algorithmsIndex[name]

		if i then
			algorithms[i] = a
		else
			table_insert(algorithms, a)

			algorithmsIndex[name] = #algorithms
		end

		if not defaultAlgorithm then
			defaultAlgorithm = a
			defaultAlgorithmName = name
		end

		return a
	end
end

do
	local setmetatable = setmetatable
	local getmetatable = getmetatable
	function core:FlushPriceData()
		self.auctionPrice 			= setmetatable({}, getmetatable(self.auctionPrice))
	end
end

function core:RegisterPricingSupport(addonName, test, init)
--~ 	Debug("RegisterPricingSupport", addonName, test, init)
	
	local t = {}

	t.name = addonName
	t.Test = test
	t.Init = init

	table_insert(pricingSupportList, t)

--~ 	if test() then
--~ 		init()
--~ 	end
end

function core:GetAlgorithmFunction(name)
	if name then
		local index = algorithmsIndex[name]
		if index then
			return algorithms[index].func, algorithms[index].name
		end
	end

	if defaultAlgorithm then
		return defaultAlgorithm.func, defaultAlgorithm.name
	else
		return function () return 0 end, "no auction support"
	end
end

function core:CanGetPrices()
	return defaultAlgorithm and true or false
end


do
	local math_floor = math.floor
	------------------------------------------------------------------
	function core:Round(num, zeros)										--
	-- zeroes is the number of decimal places. eg 1=*.*, 3=*.***	--
	------------------------------------------------------------------
		return math_floor( num * 10 ^ (zeros or 0) + 0.5 ) / 10 ^ (zeros or 0)
	end
end

--Cache prices in a metatable to reduce CPU usage. It'll get reset when addon's reset or when a auction tells us to flush our data.
core.auctionPrice = setmetatable({}, {
	__index = function(t,itemID)
--~ 		t[itemID] = core:GetAuctionValue(itemID, true)
		local auctionFunc, costName = core:GetAlgorithmFunction(core.db.profile.pricingAlgorithm)
		local value = auctionFunc(itemID) or 0
		if value == 0 then
			value = core.vendorValue(itemID)
		end
		t[itemID] = core:Round(value)

		return t[itemID]
	end
})

function core.vendorValue() return 0 end


do
	local GetCoinTextureString = GetCoinTextureString
	local GetCoinText = GetCoinText
	
	function core:GetValueString(itemLink, stack)
		local value, to = 0, "";
		
		value = self.auctionPrice[ core.iID[itemLink] ]
		
		if value and value > 0 then
--~ 			Debug("GetValueString", itemLink, self.db.profile.pricingAlgorithm)
			local msg = self.db.profile.moneyMessage
			
			if value == core.vendorValue(core.iID[itemLink]) then
				msg = msg:gsub("!ALGORITHM", L["Vendor Value"])
			else
				msg = msg:gsub("!ALGORITHM", self.db.profile.pricingAlgorithm)
			end
			msg = msg:gsub("!VALUE", self:CopperToCoins(value * (stack or 1)))
			return msg
		end
		return "";
	end
end

do
	local tonumber = tonumber
	local math_floor = math.floor
	local COPPER_AMOUNT_SYMBOL = COPPER_AMOUNT_SYMBOL
	local GOLD_AMOUNT_SYMBOL = GOLD_AMOUNT_SYMBOL
	local SILVER_AMOUNT_SYMBOL = SILVER_AMOUNT_SYMBOL
	
	local iconpath = "Interface\\MoneyFrame\\UI-"
	function core:CopperToCoins(copper)
	--~ 	Debug("CopperToCoins A", copper, g, s, c)
		copper = tonumber(copper or 0)
		local g = math_floor(copper / 10000)
		local s = math_floor(copper % 10000 / 100)
		local c = copper % 100
	
	--~ 	Debug("CopperToCoins B", copper, g, s, c)
		
		local text = "";
		if self.db.profile.colourBlindMode == true then
			if g ==0 and s ==0 then
				text = text.."|cFFFF6600" .. c .. COPPER_AMOUNT_SYMBOL.. "|r"
			else
				if g > 0 then
					text = text.."|cFFFFFF00" .. g .. GOLD_AMOUNT_SYMBOL .. "|r"
				end
				if s > 0 then
					text = text.."|cFFCCCCCC" .. s .. SILVER_AMOUNT_SYMBOL .."|r"
				end
				if c > 0 then
					text = text.."|cFFFF6600" .. c .. COPPER_AMOUNT_SYMBOL .. "|r"
				end
			end
			
		else
			if g ==0 and s ==0 then
				text = text.."|cFFFF6600" .. c .. "|r|T"..iconpath.."CopperIcon:0|t"
			else
				if g > 0 then
					text = text.."|cFFFFFF00" .. g .. "|r|T"..iconpath.."GoldIcon:0|t"
				end
				if s > 0 then
					text = text.."|cFFCCCCCC" .. s .. "|r|T"..iconpath.."SilverIcon:0|t"
				end
				if c > 0 then
					text = text.."|cFFFF6600" .. c .. "|r|T"..iconpath.."CopperIcon:0|t"
				end
			end
		end
		return text;
	end
end
